/**
 * REST demo programs
 */
package demo.rest;

import java.io.File;
import javax.ws.rs.core.MediaType;

import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.methods.PostMethod;
import org.apache.commons.httpclient.methods.PutMethod;
import org.apache.commons.httpclient.methods.multipart.FilePart;
import org.apache.commons.httpclient.methods.multipart.MultipartRequestEntity;
import org.apache.commons.httpclient.methods.multipart.Part;
import org.apache.commons.httpclient.methods.multipart.StringPart;

/********************************************************************
 * USED FOR THE Wink RestClient IMPLEMENTATION
 *
import javax.ws.rs.core.MultivaluedMap;
import org.apache.wink.client.ClientResponse;
import org.apache.wink.client.Resource;
import org.apache.wink.common.internal.utils.MediaTypeUtils;

import org.apache.wink.common.model.multipart.OutPart;
import org.apache.wink.common.model.multipart.BufferedOutMultiPart;
import org.apache.wink.common.internal.MultivaluedMapImpl;
********************************************************************/

/**
 * Load a static resource file. This program extends demo.rest.BaseClient.java
 * which in turn makes use of demo.rest.BaseUtil.java.
 * <p>
 * Note:<br/>
 * This implementation uses the Apache commons HttpClient, instead of the
 * Apache wink RestClient. The RestClient is generally more straightforward, 
 * but multipart requests currently do not work when made using that client.
 *      
 * @author Eric Armstrong     
 */
public class StaticResourceUploadClient extends BaseClient
{
  	
  public void execute(String cookieString)
  {
    String url = baseUrl + "/staticResource";
    //Use this for an update:
    // String url = baseUrl + "/staticResource/{id}";     // <--[1]
    String xml = "<platform>"
               + "<staticResource>"
                  + "<name>upload_test</name>"
                  + "<description>Test an API-driven upload</description>"  
               + "</staticResource>"
               + "</platform>";    
    try
    {
       
/*****************************************************************************
 * It would be natural to reuse the Wink client we instantiated in BaseClient,
 * but that client doesn't seem to do multipart deliveries properly.
 * So, here, we use HttpClient instead. (The good news is that the code is
 * simpler for multipart in HttpClient).
 * For more, on the Wink issue, see Case #2010-12-02-16879
 *   at https://na.longjump.com/networking/Service?t=218&id=1723907960
 *         
      // Create a multipart request
      BufferedOutMultiPart mp = new BufferedOutMultiPart();

      // Add XML part
      OutPart xp = new OutPart();
      MultivaluedMap<String,String> headers = 
          new MultivaluedMapImpl<String,String>();
      headers.add("name", "xml_part");
      xp.setHeaders(headers);
      xp.setContentType(MediaType.APPLICATION_XML);
      xp.setBody(xml);
      mp.addPart(xp);
      
      // Read the file content here. For a binary file, encode the contents.                              
      File file = new File("C:/testfiles/upload_test.txt"); 
      String file_content = "Test text\n";   // test content 

      // Add file part
      OutPart fp = new OutPart();
      headers = new MultivaluedMapImpl<String,String>();
      headers.add("name", "file_part");
      headers.add("filename", file.getName());
      fp.setHeaders(headers);
      fp.setContentType(MediaType.TEXT_PLAIN);
         //Use BaseUtil.mediaType() to set type based on file extension
      fp.setBody(file_content);     
      mp.addPart(fp);
      
      Resource resource = getResource(url);
      resource.header("Cookie", cookieString);                        <----
      resource.contentType(MediaTypeUtils.MULTIPART_MIXED);
      resource.accept(MediaType.APPLICATION_XML);
      ClientResponse response = resource.post(ClientResponse.class, mp);
      echoResponse(response.getEntity(String.class));
****************************************************************************/      
      // File part
      File file = new File("C:/testfiles/upload_test.txt"); 
      FilePart fp = new FilePart("file_part", file);
      fp.setContentType(MediaType.TEXT_PLAIN);            // <--[3]
         //Use BaseUtil.mediaType() to set type based on file extension
 
      // XML part
      StringPart sp = new StringPart("xml_part", xml);
      sp.setContentType(MediaType.APPLICATION_XML);        

      // Multipart wrapper
      final Part[] part = {fp, sp};        

      // Create the request
      HttpClient httpClient = new HttpClient();
      PostMethod httpMethod = new PostMethod(url);        // <--[2]
        // Use PutMethod for an update      

      // Set the response type (xml) and the session cookie. 
      //(Had we logged in with HttpClient, the cookie would be handled
      // automatically. But since we let BaseClient do the log in and log out,
      // we need to tack on the cookie string here.) 
      httpMethod.addRequestHeader("Cookie", cookieString);
      httpMethod.addRequestHeader("Accept", MediaType.APPLICATION_XML);
      httpMethod.setRequestEntity(
        new MultipartRequestEntity(part, httpMethod.getParams())
      );
 
      // Check status & echo response
      int status  = httpClient.executeMethod(httpMethod); 
      String responseXml = httpMethod.getResponseBodyAsString();
      if ((status == 200) || (status == 201))
      {
        // 200=Successful GET, PUT, or DELETE. 201=Successful POST.       
        echoResponse(httpMethod.getResponseBodyAsStream());
      }
      else
      {
        echoStatus(status, responseXml);
      }
    }
    catch (Exception e)
    {
      System.out.println(e.getMessage());
      e.printStackTrace();
    }
  }
	
  public static void main(String[] args) 
  {
    StaticResourceUploadClient client = new StaticResourceUploadClient();
    String cookieString = client.login(); 
    client.execute(cookieString);
    client.logout();
  }	
}
